@echo off
rem
rem ------------------------------------------------------------------------------
rem backup-sd.bat - 21-January-2013
rem
rem  based off of AndroidForums.com's EarlyMon's backup-sd.sh script
rem  converted to a Windows .bat file by AndroidForums.com's scary alien
rem
rem credit for FILTER method to find cards goes to -
rem   http://renzhi.ca/2012/02/03/how-to-list-all-sd-cards-on-android/ 
rem
rem ------------------------------------------------------------------------------
rem change log:
rem
rem  03-December-2013:
rem
rem    1. make tmp_work files referenced by adb unique so there are no open file conflicts
rem       (thanks to AF member iav for this suggestion! :)
rem
rem    2. add an 'adb kill-server' at the end of the script to make sure there are no
rem       files still left-open by adb; thanks to AF's rocketmonkeys for this :)
rem
rem    3. a few other minor tweaks
rem
rem ------------------------------------------------------------------------------
rem
rem run the .bat file with a first parameter/argument of "debug" to enable
rem display of helpful debugging information
rem
set debug=%1%
rem
set pwd=%~dp0
rem point to our present working directory
cd "%pwd%"
set save_path=%pwd%android_storage_backups
rem
set adb=%pwd%adb.exe
echo #
echo # ----------------------------------------------------------------------
echo # AndroidForums.com backup-sd.bat (version 03-December-2013)
echo #
echo #  Instructions:
echo #  
echo #  1. make sure you have the proper adb USB drivers installed
echo #  2. make sure you have USB debugging enabled
echo #  3. make sure you device is connected to your computer via a USB cable
echo #  4. if applicable, ensure your SD card is mounted as 'Charge only'
echo #     instead of 'USB Mass Storage'
echo #
echo # ----------------------------------------------------------------------
echo # 
echo # Please reboot your device, then press the [Enter] key to 
echo # start the backup or [control-c] to quit:
echo #
pause
rem
rem get this device's serial number (start with a fresh adb server, too)
rem
"%adb%" get-serialno > tmp_work0
set/p snum= < tmp_work0
rem
rem okay, let's get the internal SD card name
rem
rem define the location of our special Windows utility
rem
set afutil=%pwd%afutil.exe
rem
rem okay, let's get a list of the SD cards reference on the device (via it's mount command)
rem
rem save the mount-output for further parsing below
rem
"%adb%" shell mount > tmp_mountinfo
rem 
rem first, get a unique list of SD cards listed in the mount table
rem
rem  we have to determine if the mount info has "on" in the second field or not
rem
"%afutil%" -F tmp_mountinfo -e "vfat|/mnt" -f 2 -h 1 > tmp_work1
set/p strwork= < tmp_work1
rem
if "%strwork%" == "on" (
 rem pull from field 3
 "%afutil%" -F tmp_mountinfo -e "vfat|/mnt" | "%afutil%" -e "/dev/block/vold" | "%afutil%" -v -e "/mnt/asec|/mnt/secure" -f 3 -u > tmp_cardwork
 "%afutil%" -F tmp_mountinfo -e "/dev/fuse" -f 3 -h 1 >> tmp_cardwork
)
if "%strwork%" neq "on" (
 rem pull from field 2
 "%afutil%" -F tmp_mountinfo -e "vfat|/mnt" | "%afutil%" -e "/dev/block/vold" | "%afutil%" -v -e "/mnt/asec|/mnt/secure" -f 2 -u > tmp_cardwork
 "%afutil%" -F tmp_mountinfo -e "/dev/fuse" -f 2 -h 1 >> tmp_cardwork
)
rem
rem make the whole list unique (not sorted, mind you)
rem
"%afutil%" -F tmp_cardwork -u > tmp_cards
rem
rem get a count of the number of unique entries we found above
rem
"%afutil%" -F tmp_cards -c > tmp_work2
set/p nc= < tmp_work2
if "%debug%" == "debug" (
 echo #
 echo # pwd:       %pwd%
 echo # save_path: %save_path%
 echo # adb:       %adb%
 echo # snum:      %snum%
 echo # nc:        %nc%
)
rem
rem initialize the card2 variable
rem
set card1=none
set card2=none
rem
if "%nc%" == "0" (
 echo #
 echo # Hmm, I'm sorry, but I can't properly identify your SD card[s]
 echo # 
 echo # [script is exiting...press [Enter] key to continue]
 pause
 exit 1
)
if "%nc%" == "1" (
 "%afutil%" -F tmp_cards -r 1 > tmp_card1
 set/p card1= < tmp_card1
)
if "%nc%" == "2" (
 "%afutil%" -F tmp_cards -r 1 > tmp_card1
 set/p card1= < tmp_card1
 "%afutil%" -F tmp_cards -r 2 > tmp_card2
 set/p card2= < tmp_card2
)
if "%nc%" GTR "2" (
 echo #
 echo # Hmm, I've apparently seen more than two SD cards...not sure which to use.
 echo # 
 echo # [script is exiting...press [Enter] key to continue]
 pause
 exit 1
)
rem
if "%debug%" == "debug" (
 echo # card1:     %card1%
 echo # card2:     %card2%
)
rem
rem snag the current date in YMD format courtesy of http://snipplr.com/view/45069/
rem
for /F "tokens=1* delims= " %%A in ('date /T') do set CDATE=%%B
for /F "tokens=1,2 eol=/ delims=/ " %%A in ('date /T') do set d_mm=%%B
for /F "tokens=1,2 delims=/ eol=/" %%A in ('echo %CDATE%') do set d_dd=%%B
for /F "tokens=2,3 delims=/ " %%A in ('echo %CDATE%') do set d_yyyy=%%B
rem
rem now, parse the current time (hh and mm)
rem
for /F "tokens=1  delims=:" %%A in ('time /T') do set t_hh=%%A
for /F "tokens=2  delims=:" %%A in ('time /T') do set work=%%A
for /F "tokens=1  delims= " %%A in ('echo %work%') do set t_mm=%%A
set wdate=%d_yyyy%%d_mm%%d_dd%
set wtime=%t_hh%%t_mm%
if "%debug%" == "debug" (
 echo # wdate:     %wdate%
 echo # wtime:     %wtime%
)
set target1=%save_path%\%snum%_%wdate%_%wtime%
set target2=%save_path%\%snum%_%wdate%_%wtime%_2nd
if "%debug%" == "debug" (
 echo # target1:   %target1%
 echo # target2:   %target2%
 echo # 
 echo # okay, the real rubber hits the road here/now...
 echo # 
)
rem first, create our backup directory
rem
mkdir "%target1%" 2>NUL
rem
rem next, do the pull from the primary SD card
rem
if "%debug%" == "debug" (
 echo #
 echo [about to do the actual backup--press [Enter] to continue]
 echo [backup command: "%adb%" pull %card1% "%target1%"]
 echo #
 pause
)
date/t > tmp_bkup_start_date
time/t > tmp_bkup_start_time
echo # here we go...
"%adb%" pull %card1% "%target1%"
rem
rem next, let's cleanup the LOST.DIR from the backup
rem
cd "%target1%"
rmdir/s/q LOST.DIR 2>NUL
cd ..
rem
:check_for_secondary_backup
rem do we have a secondary SD card to backup?
rem
if "%nc%" == "2" (
  if "%card2%" neq "none" (
    if "%card2%" neq "%card1%" goto :backup_secondary_sdcard
    )
  )
)
echo # no secondary SD card found needing to be backed-up
goto :all_done
rem
:backup_secondary_sdcard
mkdir "%target2%" 2>NUL
"%adb%" pull %card2% "%target2%"
cd "%target2%"
rmdir/s/q LOST.DIR 2>NUL
cd ..
rem
:all_done
cd "%pwd%"
date/t > tmp_bkup_end_date
time/t > tmp_bkup_end_time
set/p bkup_start_date= < tmp_bkup_start_date
set/p bkup_start_time= < tmp_bkup_start_time
set/p bkup_end_date= < tmp_bkup_end_date
set/p bkup_end_time= < tmp_bkup_end_time
echo #
echo # backup started:   %bkup_start_date% %bkup_start_time%
echo # backup completed: %bkup_end_date% %bkup_end_time%
echo #
echo # all done! [press [Enter] to exit the script]
"%adb%" kill-server
rem if we are not in debugging mode, then let's cleanup our files
if "%debug%" neq "debug" (
 del tmp_bkup_end_date
 del tmp_bkup_end_time
 del tmp_bkup_start_date
 del tmp_bkup_start_time
 del tmp_card1
 del tmp_cards
 del tmp_cardwork
 del tmp_mountinfo
 del tmp_work0
 del tmp_work1
 del tmp_work2
)
pause
